@@ -1,20 +1,24 @@
+"""
+TreeSAPP command line.
+"""
 import sys
 import argparse
 import logging
 
-from .commands import (create, evaluate, abundance, assign, update, info, train, layer, purity)
+from .commands import (create, evaluate, abundance, assign, update, info, train, layer, purity, package)
 
 usage = """
 treesapp <command> [<args>]
 ** Commands include:
-create         Create a reference package for a new gene, domain or orthologous group
-evaluate       Evaluate the classification performance using clade exclusion analysis
-purity         Characterize the sequences in a reference package using a curated database
 abundance      Calculate abundance measures for classified sequences 
 assign         Classify query [protein|genomic] sequences using reference packages
-layer          Layer extra annotation information on classifications with iTOL colours-style file(s)  
+create         Create a reference package for a new gene, domain or orthologous group
+evaluate       Evaluate the classification performance using clade exclusion analysis
+layer          Layer extra annotation information on classifications with iTOL colours-style file(s)
+package        Facilitate operations on reference packages
+purity         Characterize the sequences in a reference package using a curated database
+train          Train a reference package's model used for correcting taxonomic rank classifications
 update         Update an existing reference package with sequences found in a `classify` run
-train          Train a reference package's model used for correcting taxonomic rank classifications 
 ** Other commands:
 info           Display TreeSAPP version and other information.
 Use '-h' to get subcommand-specific help, e.g.
@@ -30,7 +34,8 @@ def main():
                 "info": info,
                 "train": train,
                 "layer": layer,
-                "purity": purity}
+                "purity": purity,
+                "package": package}
     parser = argparse.ArgumentParser(description='Phylogenetic classification of biological sequences')
     parser.add_argument('command', nargs='?')
     args = parser.parse_args(sys.argv[1:2])
