@@ -2,151 +2,244 @@ import sys
 import os
 import time
 import re
-import glob
 import logging
 from shutil import copy
 
-from treesapp.external_command_interface import launch_write_command, setup_progress_bar
-from treesapp.classy import CommandLineFarmer
-from .fasta import read_fasta_to_dict
-from .utilities import remove_dashes_from_msa
+from tqdm import tqdm
+
+from treesapp.external_command_interface import launch_write_command, CommandLineFarmer
+from treesapp.fasta import read_fasta_to_dict
 
 
-def construct_tree(executables: dict, molecule: str, multiple_alignment_file: str,
-                   tree_output_dir, tree_file, tree_prefix, args):
+def select_model(molecule: str, raxml_model=None) -> str:
+    """
+    Eventually this function will be a wrapper for ModelTest-ng or IQTree's ModelFinder.
+
+    :param raxml_model: An optional string with the RAxML-NG model used
+    :param molecule: A string indicating the molecule-type of the reference package: 'rrna', 'prot' or 'dna'
+    :return: A RAxML-ng and EPA-ng compatible string representing the substitution model
+    """
+    if raxml_model:
+        evo_model = raxml_model
+    elif molecule == "prot":
+        evo_model = "LG+G4"
+    elif molecule == "rrna" or molecule == "dna":
+        evo_model = "GTR+G"
+    else:
+        logging.error("A substitution model could not be specified from the molecule argument '{}'.\n".format(molecule))
+        sys.exit(13)
+    return evo_model
+
+
+def model_parameters(raxml_exe: str, ref_msa: str, tree_file: str, output_prefix: str, model: str, threads=2) -> str:
+    """
+    Wrapper function for RAxML-ng's `evaluate` sub-command that generates a file to be used by EPA-ng.
+    This file, in reality, is part of a reference package as it is reference tree and MSA dependent
+    and will need to be provided for phylogenetic placement.
+
+    :param raxml_exe: Path to the raxml-ng executable
+    :param ref_msa: Path to the reference package's multiple sequence alignment (FASTA)
+    :param tree_file: Path to the reference package's tree (Newick)
+    :param output_prefix: Prefix path for the output files
+    :param model: Substitution model (and other parameters e.g. Gamma model of rate heterogeneity) used to build tree
+    :param threads: The number of threads that should be used by RAxML-NG
+    :return: Path to the bestModel file that can be used by epa-ng for phylogenetic placement
+    """
+    output_prefix += "_evaluate"
+    model_params_file = output_prefix + ".raxml.bestModel"
+    model_eval_cmd = [raxml_exe, "--evaluate"]
+    model_eval_cmd += ["--msa", ref_msa]
+    model_eval_cmd += ["--tree", tree_file]
+    model_eval_cmd += ["--prefix", output_prefix]
+    model_eval_cmd += ["--model", model]
+    model_eval_cmd += ["--threads", str(threads)]
+    model_eval_cmd += ["--seed", str(12345)]
+    model_eval_cmd += ["--workers", str(1)]
+    model_eval_cmd.append("--force")
+
+    logging.debug("Evaluating phylogenetic tree with RAxML-NG... ")
+    stdout, returncode = launch_write_command(model_eval_cmd)
+    logging.debug("done.\n")
+
+    if returncode != 0:
+        logging.error("{} did not complete successfully! Look in {}_info.txt for an error message.\n"
+                      "RAxML-NG command used:\n{}\n".format(raxml_exe, output_prefix, ' '.join(model_eval_cmd)))
+        sys.exit(13)
+
+    return model_params_file
+
+
+def bootstrap_tree_raxml(raxml_exe: str, multiple_alignment: str, model: str, tree_prefix: str,
+                         mre=True, bootstraps=1000, num_threads=2) -> str:
+    bootstrap_cmd = [raxml_exe, "--bootstrap"]
+    bootstrap_cmd += ["--msa", multiple_alignment]
+    bootstrap_cmd += ["--model", model]
+    if mre:
+        bootstrap_cmd += ["--bs-trees", "autoMRE{%d}" % bootstraps]
+    else:
+        bootstrap_cmd += ["--bs-trees", str(bootstraps)]
+    bootstrap_cmd += ["--prefix", tree_prefix]
+    bootstrap_cmd += ["--seed", str(12345)]
+    bootstrap_cmd += ["--threads", str(num_threads)]
+
+    logging.info("Bootstrapping reference tree with RAxML-NG... ")
+    launch_write_command(bootstrap_cmd)
+    logging.info("done.\n")
+
+    bootstraps_file = tree_prefix + ".raxml.bootstraps"
+    if not os.path.isfile(bootstraps_file):
+        logging.error("Unable to find bootstrap file '%s'.\n" % bootstraps_file)
+        sys.exit(17)
+
+    return bootstraps_file
+
+
+def support_tree_raxml(raxml_exe: str, ref_tree: str, ref_msa: str, model: str, tree_prefix: str,
+                       mre=False, n_bootstraps=1000, num_threads=2) -> str:
+    bootstraps = bootstrap_tree_raxml(raxml_exe, ref_msa, model, tree_prefix, mre, n_bootstraps, num_threads)
+
+    support_cmd = [raxml_exe, "--support"]
+    support_cmd += ["--tree", ref_tree]
+    support_cmd += ["--bs-trees", bootstraps]
+    support_cmd += ["--prefix", tree_prefix]
+    support_cmd += ["--threads", str(num_threads)]
+
+    logging.info("Calculating bootstrap support for node in reference tree with RAxML-NG... ")
+    launch_write_command(support_cmd)
+    logging.info("done.\n")
+
+    support_file = tree_prefix + ".raxml.support"
+    if not os.path.isfile(support_file):
+        logging.error("Unable to find support file '%s'.\n" % support_file)
+        sys.exit(17)
+
+    return support_file
+
+
+def construct_tree(tree_builder: str, executables: dict, evo_model: str, multiple_alignment_file: str,
+                   tree_output_dir: str, tree_prefix: str, num_threads=2) -> str:
     """
     Wrapper script for generating phylogenetic trees with either RAxML or FastTree from a multiple alignment
 
+    :param tree_builder: String indicating which phylogeny inference software is to be used. Current options are
+    FastTree and RAxML-NG
     :param executables: Dictionary containing paths to executables, crucially FastTree and RAxML
-    :param molecule: Molecule type of the sequences being used to infer the phylogeny
+    :param evo_model: The substitution model (and possible gamma rate heterogeneity) string (e.g. GTR+G4)
     :param multiple_alignment_file: Path to the multiple sequence alignment file
     :param tree_output_dir: Path to the directory where output files should be written to
-    :param tree_file: Path to write the inferred phylogenetic tree
     :param tree_prefix: Prefix to be used for the outputs
-    :param args: Command-line arguments parsed using ArgParse
+    :param num_threads: Number of threads to use (for RAxML-NG only)
     :return: Stylized name of the tree-building software used
     """
 
     # Decide on the command to build the tree, make some directories and files when necessary
-    if args.fast:
+    logging.info("Building phylogenetic tree with " + tree_builder + "... ")
+    if tree_builder == "FastTree":
+        best_tree = tree_output_dir + tree_prefix + ".FastTree.nwk"
         tree_build_cmd = [executables["FastTree"]]
-        if molecule == "rrna" or molecule == "dna":
+        if re.search(r"GTR", evo_model):
             tree_build_cmd += ["-nt", "-gtr"]
         else:
-            tree_build_cmd += ["-lg", "-wag"]
-        tree_build_cmd += ["-out", tree_file]
+            tree_build_cmd += ["-lg", "-gamma", "-cat", str(4)]
+        tree_build_cmd += ["-out", best_tree]
         tree_build_cmd.append(multiple_alignment_file)
-        tree_builder = "FastTree"
-    else:
-        tree_build_cmd = [executables["raxmlHPC"]]
-        tree_build_cmd += ["-f", "a"]
-        tree_build_cmd += ["-p", "12345"]
-        tree_build_cmd += ["-x", "12345"]
-        tree_build_cmd += ["-#", str(args.bootstraps)]
-        tree_build_cmd += ["-s", multiple_alignment_file]
-        tree_build_cmd += ["-n", tree_prefix]
-        tree_build_cmd += ["-w", tree_output_dir]
-        tree_build_cmd += ["-T", str(args.num_threads)]
 
-        if args.raxml_model:
-            tree_build_cmd += ["-m", args.raxml_model]
-        elif args.molecule == "prot":
-            tree_build_cmd += ["-m", "PROTGAMMAAUTO"]
-        elif args.molecule == "rrna" or molecule == "dna":
-            tree_build_cmd += ["-m", "GTRGAMMA"]
-        else:
-            logging.error("A substitution model could not be specified with the 'molecule' argument: " + args.molecule)
-            sys.exit(13)
-        tree_builder = "RAxML"
-
-    # Ensure the tree from a previous run isn't going to be over-written
-    if not os.path.exists(tree_output_dir):
-        os.makedirs(tree_output_dir)
-    else:
-        logging.error(tree_output_dir + " already exists from a previous run! " +
-                      "Please delete or rename it and try again.\n")
-        sys.exit(13)
-
-    logging.info("Building phylogenetic tree with " + tree_builder + "... ")
-    if args.fast:
-        stdout, returncode = launch_write_command(tree_build_cmd, True)
-        with open(tree_output_dir + os.sep + "FastTree_info." + tree_prefix, 'w') as fast_info:
+        stdout, returncode = launch_write_command(tree_build_cmd)
+        with open(tree_output_dir + tree_prefix + ".FastTree.log", 'w') as fast_info:
             fast_info.write(stdout + "\n")
+    elif tree_builder == "RAxML-NG":
+        best_tree = tree_output_dir + tree_prefix + ".raxml.bestTree"
+        tree_build_cmd = [executables["raxml-ng"], "--search"]
+        tree_build_cmd += ["--prefix", tree_output_dir + tree_prefix]
+        tree_build_cmd += ["--msa", multiple_alignment_file]
+        tree_build_cmd += ["--model", evo_model]
+        # tree_build_cmd += ["--msa-format", "PHYLIP"]  # File isn't read properly with this parameter, use auto-detect
+        tree_build_cmd += ["--seed", str(12345)]
+        tree_build_cmd += ["--threads", str(num_threads)]
+        # tree_build_cmd += ["--tree", "rand{1},pars{1}"]  # For debugging, alternatively could use '--search1'
+
+        stdout, returncode = launch_write_command(tree_build_cmd)
     else:
-        stdout, returncode = launch_write_command(tree_build_cmd, False)
+        logging.error("Unrecognized software '{}'.\n".format(tree_builder))
+        sys.exit(5)
+
     logging.info("done.\n")
+    logging.debug(stdout + "\n")
 
     if returncode != 0:
-        logging.error(tree_builder + " did not complete successfully! " +
-                      "Look in " + tree_output_dir + os.sep +
-                      tree_builder + "_info." + tree_prefix + " for an error message.\n" +
-                      tree_builder + " command used:\n" + ' '.join(tree_build_cmd) + "\n")
+        logging.error("{0} did not complete successfully! Look in {1} for an error message.\n"
+                      "{0} command used:\n{2}\n".format(tree_builder,
+                                                        tree_output_dir + '.'.join([tree_prefix, tree_builder, "log"]),
+                                                        ' '.join(tree_build_cmd)))
         sys.exit(13)
 
-    return tree_builder
+    return best_tree
 
 
-def launch_evolutionary_placement_queries(executables, tree_dir, phy_files, marker_build_dict, output_dir, num_threads):
+def launch_evolutionary_placement_queries(executables: dict, split_msa_files: dict,
+                                          refpkg_dict: dict, output_dir: str,
+                                          num_threads: int) -> None:
     """
-    Run EPA through RAxML using Phylip files containing the reference and query sequences, and the reference trees
+    Run EPA-ng using FASTA files containing the reference and query sequences, and the reference trees
 
-    :param executables:
-    :param tree_dir:
-    :param phy_files:
-    :param marker_build_dict:
-    :param output_dir:
-    :param num_threads:
+    :param executables: Dictionary of executables where executable name strings are keys and paths are values
+    :param split_msa_files: Dictionary of TreeSAPP refpkg code (denominator) keys indexing a list of
+     namedtuple instances called MSAs. Each instance has 'ref' and 'query' variables referring to the
+     MSA files (FASTA formatted) with aligned reference sequences and aligned query sequences, respectively.
+    :param refpkg_dict: Dictionary of ReferencePackage instances indexed by their TreeSAPP refpkg code (denominator)
+    :param output_dir: Path to write the EPA-ng outputs
+    :param num_threads: Number of threads to use during placement
     :return: None
     """
-    logging.info("Running RAxML... coffee?\n")
+    logging.info("Running EPA... ")
 
     start_time = time.time()
 
-    raxml_calls = 0
+    epa_calls = 0
     # Maximum-likelihood sequence placement analyses
-    denominator_reference_tree_dict = dict()
-    for denominator in sorted(phy_files.keys()):
-        if not isinstance(denominator, str):
-            logging.error(str(denominator) + " is not string but " + str(type(denominator)) + "\n")
+    for refpkg_name in sorted(split_msa_files.keys()):
+        if not isinstance(refpkg_name, str):
+            logging.error(str(refpkg_name) + " is not string but " + str(type(refpkg_name)) + "\n")
             raise AssertionError()
-        # Establish the reference tree file to be used for this contig
-        ref_marker = marker_build_dict[denominator]
-        reference_tree_file = tree_dir + os.sep + ref_marker.cog + '_tree.txt'
-        if denominator not in denominator_reference_tree_dict.keys():
-            denominator_reference_tree_dict[denominator] = reference_tree_file
-        for phy_file in phy_files[denominator]:
-            query_name = re.sub("_hmm_purified.phy.*$", '', os.path.basename(phy_file))
-            query_name = re.sub(marker_build_dict[denominator].cog, denominator, query_name)
-            raxml_evolutionary_placement(executables["raxmlHPC"], reference_tree_file, phy_file,
-                                         ref_marker.model, output_dir, query_name, num_threads)
-            raxml_calls += 1
+        ref_pkg = refpkg_dict[refpkg_name]
+        for split_msa in split_msa_files[refpkg_name]:
+            query_name = re.sub("_queries.mfa", '', os.path.basename(split_msa.query))
+            query_name = re.sub(ref_pkg.prefix, refpkg_name, query_name)
+            # Find the query names
+            raxml_evolutionary_placement(executables["epa-ng"], ref_pkg.f__tree, split_msa.ref, ref_pkg.f__model_info,
+                                         split_msa.query, query_name, output_dir, num_threads)
+            epa_calls += 1
 
     end_time = time.time()
     hours, remainder = divmod(end_time - start_time, 3600)
     minutes, seconds = divmod(remainder, 60)
-    logging.debug("\tRAxML time required: " +
+    logging.info("done.\n")
+
+    logging.debug("\tEPA-ng time required: " +
                   ':'.join([str(hours), str(minutes), str(round(seconds, 2))]) + "\n")
-    logging.debug("\tRAxML was called " + str(raxml_calls) + " times.\n")
+    logging.debug("\tEPA-ng was called " + str(epa_calls) + " times.\n")
 
     return
 
 
-def raxml_evolutionary_placement(raxml_exe: str, reference_tree_file: str, multiple_alignment: str, model: str,
-                                 output_dir: str, query_name: str, num_threads=2):
+def raxml_evolutionary_placement(epa_exe: str, refpkg_tree: str, refpkg_msa: str, refpkg_model: str,
+                                 query_msa: str, query_name: str, output_dir: str, num_threads=2):
     """
-    A wrapper for RAxML's evolutionary placement algorithm (EPA)
+    A wrapper for evolutionary placement algorithm (EPA) next-generation
         1. checks to ensure the output files do not already exist, and removes them if they do
-        2. ensures the output directory is an absolute path, satisfying RAxML
-        3. Runs RAxML with the provided parameters
+        2. ensures the output directory is an absolute path, satisfying EPA
+        3. Runs EPA with the provided parameters
         4. Renames the files for consistency in TreeSAPP
-    :param raxml_exe: Path to the RAxML executable to be used
-    :param reference_tree_file: The reference tree for evolutionary placement to operate on
-    :param multiple_alignment: Path to a multiple alignment file containing reference and query sequences
-    :param model: The substitution model to be used by RAxML e.g. PROTGAMMALG, GTRCAT
-    :param output_dir: Path to write the EPA outputs
+
+    :param epa_exe: Path to the EPA-ng executable to be used
+    :param refpkg_tree: The reference tree for evolutionary placement to operate on
+    :param refpkg_msa: The reference multiple sequence alignment for the reference package (FASTA)
+    :param refpkg_model: The substitution model to be used by EPA e.g. PROTGAMMALG, GTRCAT
+    :param query_msa: Path to a multiple alignment file containing aligned query sequences (FASTA)
     :param query_name: Prefix name for all of the output files
+    :param output_dir: Path to write the EPA outputs
     :param num_threads: Number of threads EPA should use (default = 2)
-    :return: A dictionary of files written by RAxML's EPA that are used by TreeSAPP. For example epa_files["jplace"]
+    :return: A dictionary of files written by EPA-ng that are used by TreeSAPP. For example epa_files["jplace"]
     """
     epa_files = dict()
     ##
@@ -159,71 +252,59 @@ def raxml_evolutionary_placement(raxml_exe: str, reference_tree_file: str, multi
     if output_dir[-1] != os.sep:
         output_dir += os.sep
 
-    if model is None:
+    if refpkg_model is None:
         logging.error("No substitution model provided for evolutionary placement of " + query_name + ".\n")
         raise AssertionError()
 
     # Determine the output file names, and remove any pre-existing output files
-    if not isinstance(reference_tree_file, str):
-        logging.error(str(reference_tree_file) + " is not string but " + str(type(reference_tree_file)) + "\n")
+    if not isinstance(refpkg_tree, str):
+        logging.error(str(refpkg_tree) + " is not string but " + str(type(refpkg_tree)) + "\n")
         raise AssertionError()
 
-    if len(reference_tree_file) == 0:
-        logging.error("Could not find reference tree for " + query_name + " to be used by EPA.\n")
+    if len(refpkg_tree) == 0:
+        logging.error("Could not find reference tree for " + query_name + " to be used by EPA-ng.\n")
         raise AssertionError()
 
-    # This is the final set of files that will be written by RAxML's EPA algorithm
-    epa_files["stdout"] = output_dir + query_name + '_RAxML.txt'
-    epa_info = output_dir + 'RAxML_info.' + query_name
-    epa_files["info"] = output_dir + query_name + '.RAxML_info.txt'
-    epa_labelled_tree = output_dir + 'RAxML_labelledTree.' + query_name
-    epa_tree = output_dir + 'RAxML_originalLabelledTree.' + query_name
-    epa_files["tree"] = output_dir + query_name + '.originalRAxML_labelledTree.txt'
-    epa_classification = output_dir + 'RAxML_classification.' + query_name
-    epa_files["classification"] = output_dir + query_name + '.RAxML_classification.txt'
-    epa_files["jplace"] = output_dir + "RAxML_portableTree." + query_name + ".jplace"
-    epa_entropy = output_dir + "RAxML_entropy." + query_name
-    epa_weights = output_dir + "RAxML_classificationLikelihoodWeights." + query_name
+    # This is the final set of files that will be written by EPA-ng
+    epa_files["stdout"] = output_dir + query_name + '_EPA.txt'
+    epa_info = output_dir + 'epa_info.log'
+    epa_files["info"] = output_dir + query_name + '.EPA_info.txt'
+    epa_jplace = output_dir + "epa_result.jplace"
+    epa_files["jplace"] = output_dir + "epa_result." + query_name + ".jplace"
 
-    for raxml_file in [epa_info, epa_labelled_tree, epa_tree, epa_classification, epa_entropy, epa_weights]:
+    for raxml_file in [epa_info, epa_jplace]:
         try:
             os.remove(raxml_file)
         except OSError:
             pass
 
-    # Set up the command to run RAxML
-    raxml_command = [raxml_exe,
-                     '-m', model,
-                     '-T', str(int(num_threads)),
-                     '-s', multiple_alignment,
-                     '-t', reference_tree_file,
-                     '-G', str(0.2),
-                     "--epa-prob-threshold=" + str(0.10),
-                     '-f', 'v',
-                     '-n', query_name,
-                     '-w', output_dir,
-                     '>', epa_files["stdout"]]
-    launch_write_command(raxml_command)
+    # Set up the command to run EPA-ng
+    epa_command = [epa_exe,
+                   '-s', refpkg_msa,
+                   '-t', refpkg_tree,
+                   '-q', query_msa,
+                   "--model", refpkg_model,
+                   "--no-pre-mask",
+                   "--dyn-heur", str(0.9),
+                   # "--fix-heur", str(0.2),
+                   "--preserve-rooting", "on",
+                   "--filter-min-lwr", str(0.01),
+                   "--outdir", output_dir,
+                   '-T', str(num_threads),
+                   '>', epa_files["stdout"]]
+    launch_write_command(epa_command)
 
     # Rename the RAxML output files
     if os.path.exists(epa_info):
         copy(epa_info, epa_files["info"])
         os.remove(epa_info)
-    if os.path.exists(epa_classification):
-        copy(epa_classification, epa_files["classification"])
-        os.remove(epa_classification)
-    if os.path.exists(epa_tree):
-        copy(epa_tree, epa_files["tree"])
-        os.remove(epa_tree)
+    if os.path.exists(epa_jplace):
+        copy(epa_jplace, epa_files["jplace"])
+        os.remove(epa_jplace)
     else:
         logging.error("Some files were not successfully created for " + query_name + "\n" +
                       "Check " + epa_files["stdout"] + " for an error!\n")
         sys.exit(3)
-    # Remove useless files
-    if os.path.exists(epa_labelled_tree):
-        os.remove(epa_labelled_tree)
-        os.remove(epa_weights)
-        os.remove(epa_entropy)
 
     return epa_files
 
@@ -361,9 +442,12 @@ def cluster_sequences(uclust_exe, fasta_input, uclust_prefix, similarity=0.60):
     return
 
 
-def build_hmm_profile(hmmbuild_exe, msa_in, output_hmm):
+def build_hmm_profile(hmmbuild_exe: str, msa_in: str, output_hmm: str, name=None):
     logging.debug("Building HMM profile... ")
-    hmm_build_command = [hmmbuild_exe, output_hmm, msa_in]
+    hmm_build_command = [hmmbuild_exe]
+    if name:
+        hmm_build_command += ["-n", str(name)]
+    hmm_build_command += [output_hmm, msa_in]
     stdout, hmmbuild_pro_returncode = launch_write_command(hmm_build_command)
     logging.debug("done.\n")
 
@@ -389,7 +473,8 @@ def run_prodigal(args, fasta_file, output_file, nucleotide_orfs=None):
     return
 
 
-def run_hmmsearch(hmmsearch_exe: str, hmm_profile: str, query_fasta: str, output_dir: str, num_threads=2):
+def run_hmmsearch(hmmsearch_exe: str, hmm_profile: str, query_fasta: str, output_dir: str,
+                  num_threads=2, e_value=1) -> list:
     """
     Function for searching a fasta file with a profile HMM
 
@@ -398,15 +483,17 @@ def run_hmmsearch(hmmsearch_exe: str, hmm_profile: str, query_fasta: str, output
     :param query_fasta: Path to the FASTA file to be queried by the profile
     :param output_dir: Path to the directory for writing the outputs
     :param num_threads: Number of threads to be used by hmmsearch
-    :return:
+    :param e_value: report sequences <= this E-value threshold in output
+    :return: A list of domain tables created
     """
     # Find the name of the HMM. Use it to name the output file
-    rp_marker = re.sub(".hmm", '', os.path.basename(hmm_profile))
+    rp_marker = re.sub(r".hmm", '', os.path.basename(hmm_profile), flags=re.IGNORECASE)
     domtbl = output_dir + rp_marker + "_to_ORFs_domtbl.txt"
 
     # Basic hmmsearch command
     hmmsearch_command_base = [hmmsearch_exe]
     hmmsearch_command_base += ["--cpu", str(num_threads)]
+    hmmsearch_command_base += ["-E", str(e_value)]
     hmmsearch_command_base.append("--noali")
     # Customize the command for this input and HMM
     final_hmmsearch_command = hmmsearch_command_base + ["--domtblout", domtbl]
@@ -422,109 +509,61 @@ def run_hmmsearch(hmmsearch_exe: str, hmm_profile: str, query_fasta: str, output
     return [domtbl]
 
 
-def hmmsearch_orfs(hmmsearch_exe, hmm_dir, marker_build_dict, fasta_file, output_dir, num_threads=2):
+def hmmsearch_orfs(hmmsearch_exe: str, refpkg_dict: dict, fasta_file: str, output_dir: str,
+                   num_threads=2, e_value=1) -> list:
     hmm_domtbl_files = list()
     nucl_target_hmm_files = list()
     prot_target_hmm_files = list()
 
-    # Find all of the available HMM profile files
-    try:
-        os.path.isdir(hmm_dir)
-    except IOError:
-        logging.error(hmm_dir + "does not exist!")
-        sys.exit(3)
-    hmm_files = glob.glob(hmm_dir + "*.hmm")
-
-    if len(hmm_files) == 0:
-        logging.error(hmm_dir + "does not contain any files with '.hmm' extension... so no HMMs.\n")
-        sys.exit(3)
-
     # Filter the HMM files to only the target markers
-    for marker_code in marker_build_dict:
-        ref_marker = marker_build_dict[marker_code]
-        hmm_profile = hmm_dir + ref_marker.cog + ".hmm"
-        if hmm_profile not in hmm_files:
-            logging.error("Unable to locate HMM-profile for " + ref_marker.cog + "(" + marker_code + ").\n")
+    for refpkg_name in refpkg_dict:
+        refpkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
+        if not os.path.exists(refpkg.f__search_profile):
+            logging.error("Unable to locate HMM-profile for '{}'.\n".format(refpkg.prefix))
+            sys.exit(3)
         else:
-            if ref_marker.molecule == "prot":
-                prot_target_hmm_files.append(hmm_profile)
+            if refpkg.molecule == "prot":
+                prot_target_hmm_files.append(refpkg.f__search_profile)
             else:
-                nucl_target_hmm_files.append(hmm_profile)
+                nucl_target_hmm_files.append(refpkg.f__search_profile)
 
-    acc = 0.0
     logging.info("Searching for marker proteins in ORFs using hmmsearch.\n")
-    step_proportion = setup_progress_bar(len(prot_target_hmm_files) + len(nucl_target_hmm_files))
+    if logging.getLogger().disabled:
+        pbar = None
+    else:
+        pbar = tqdm(total=len(prot_target_hmm_files) + len(nucl_target_hmm_files), ncols=120)
 
     # Create and launch the hmmsearch commands iteratively.
     for hmm_file in prot_target_hmm_files:
-        # TODO: Parallelize this by allocating no more than 2 threads per process
-        hmm_domtbl_files += run_hmmsearch(hmmsearch_exe, hmm_file, fasta_file, output_dir, num_threads)
+        if pbar:
+            pbar.set_description("Processing {}".format(os.path.basename(hmm_file)))
 
-        # Update the progress bar
-        acc += 1.0
-        if acc >= step_proportion:
-            acc -= step_proportion
-            time.sleep(0.1)
-            sys.stdout.write("-")
-            sys.stdout.flush()
-    sys.stdout.write("-]\n")
+        # TODO: Parallelize this by allocating no more than 2 threads per process
+        hmm_domtbl_files += run_hmmsearch(hmmsearch_exe, hmm_file, fasta_file, output_dir, num_threads, e_value)
+
+        if pbar:
+            pbar.update()
+
+    if pbar:
+        pbar.close()
 
     return hmm_domtbl_files
 
 
-def generate_blast_database(args, fasta, molecule, prefix, multiple=True):
-    """
-
-    :param args:
-    :param fasta: File to make a BLAST database for
-    :param molecule: 'prot' or 'nucl' - necessary argument for makeblastdb
-    :param prefix: prefix string for the output BLAST database
-    :param multiple: Flag indicating the input `fasta` is a MSA. Alignment information is removed prior to makeblastdb
-    :return:
-    """
-
-    # Remove the multiple alignment information from fasta_replaced_file and write to fasta_mltree
-    blastdb_out = prefix + ".fa"
-    if multiple:
-        if blastdb_out == fasta:
-            logging.error("prefix.fa is the same as " + fasta + " and would be overwritten!\n")
-            sys.exit(13)
-        remove_dashes_from_msa(fasta, blastdb_out)
-        blastdb_in = blastdb_out
-    else:
-        blastdb_in = fasta
-
-    logging.info("Making the BLAST database for " + blastdb_in + "... ")
-
-    # Format the `makeblastdb` command
-    makeblastdb_command = [args.executables["makeblastdb"]]
-    makeblastdb_command += ["-in", blastdb_in]
-    makeblastdb_command += ["-out", blastdb_out]
-    makeblastdb_command += ["-input_type", "fasta"]
-    makeblastdb_command += ["-dbtype", molecule]
-
-    # Launch the command
-    stdout, makeblastdb_pro_returncode = launch_write_command(makeblastdb_command)
-
-    logging.info("done\n")
-
-    return stdout, blastdb_out
-
-
-def run_mafft(mafft_exe: str, fasta_in: str, fasta_out: str, num_threads):
+def run_mafft(mafft_exe: str, fasta_in: str, fasta_out: str, num_threads) -> None:
     """
     Wrapper function for the MAFFT multiple sequence alignment tool.
     Runs MAFFT using `--auto` and checks if the output is empty.
+
     :param mafft_exe: Path to the executable for mafft
     :param fasta_in: An unaligned FASTA file
     :param fasta_out: The path to a file MAFFT will write aligned sequences to
     :param num_threads: Integer (or string) for the number of threads MAFFT can use
     :return:
     """
-    mafft_align_command = [mafft_exe]
+    mafft_align_command = [mafft_exe, "--auto", "--anysymbol"]
     mafft_align_command += ["--maxiterate", str(1000)]
     mafft_align_command += ["--thread", str(num_threads)]
-    mafft_align_command.append("--auto")
     mafft_align_command += ["--randomseed", str(12345)]
     mafft_align_command += [fasta_in, '1>' + fasta_out]
     mafft_align_command += ["2>", "/dev/null"]
@@ -581,7 +620,7 @@ def get_msa_trim_command(executables, mfa_file, molecule, tool="BMGE"):
         logging.error("Unsupported file format: '" + f_ext + "'\n")
         sys.exit(5)
 
-    trimmed_msa_file = re.sub('.' + re.escape(f_ext), '-' + re.escape(tool) + ".fasta", mfa_file)
+    trimmed_msa_file = '.'.join(mfa_file.split('.')[:-1]) + '-' + re.escape(tool) + ".fasta"
     if tool == "trimAl":
         trim_command = trimal_command(executables["trimal"], mfa_file, trimmed_msa_file)
     elif tool == "BMGE":
@@ -593,13 +632,13 @@ def get_msa_trim_command(executables, mfa_file, molecule, tool="BMGE"):
     return trim_command, trimmed_msa_file
 
 
-def filter_multiple_alignments(executables, concatenated_mfa_files, marker_build_dict, n_proc=1, tool="BMGE"):
+def filter_multiple_alignments(executables, concatenated_mfa_files, refpkg_dict, n_proc=1, tool="BMGE"):
     """
     Runs BMGE using the provided lists of the concatenated hmmalign files, and the number of sequences in each file.
 
     :param executables: A dictionary mapping software to a path of their respective executable
     :param concatenated_mfa_files: A dictionary containing f_contig keys mapping to a FASTA or Phylip sequential file
-    :param marker_build_dict:
+    :param refpkg_dict: A dictionary of ReferencePackage instances indexed by their respective denominators
     :param n_proc: The number of parallel processes to be launched for alignment trimming
     :param tool: The software to use for alignment trimming
     :return: A list of files resulting from BMGE multiple sequence alignment masking.
@@ -610,14 +649,14 @@ def filter_multiple_alignments(executables, concatenated_mfa_files, marker_build
     task_list = list()
     trimmed_output_files = {}
 
-    for denominator in sorted(concatenated_mfa_files.keys()):
-        if denominator not in trimmed_output_files:
-            trimmed_output_files[denominator] = []
-        mfa_files = concatenated_mfa_files[denominator]
+    for refpkg_code in sorted(concatenated_mfa_files.keys()):
+        if refpkg_code not in trimmed_output_files:
+            trimmed_output_files[refpkg_code] = []
+        mfa_files = concatenated_mfa_files[refpkg_code]
         for concatenated_mfa_file in mfa_files:
             trim_command, trimmed_msa_file = get_msa_trim_command(executables, concatenated_mfa_file,
-                                                                  marker_build_dict[denominator].molecule, tool)
-            trimmed_output_files[denominator].append(trimmed_msa_file)
+                                                                  refpkg_dict[refpkg_code].molecule, tool)
+            trimmed_output_files[refpkg_code].append(trimmed_msa_file)
             task_list.append(trim_command)
 
     if len(task_list) > 0:
